@@ -4,31 +4,47 @@
 #                      tree already exits.
 #
 #  Subversion is a tool for revision control.
-#  See http://subversion.tigris.org for more information.
+#  See http://subversion.apache.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2000-2006, 2008 CollabNet.  All rights reserved.
+#    Licensed to the Apache Software Foundation (ASF) under one
+#    or more contributor license agreements.  See the NOTICE file
+#    distributed with this work for additional information
+#    regarding copyright ownership.  The ASF licenses this file
+#    to you under the Apache License, Version 2.0 (the
+#    "License"); you may not use this file except in compliance
+#    with the License.  You may obtain a copy of the License at
 #
-# This software is licensed as described in the file COPYING, which
-# you should have received as part of this distribution.  The terms
-# are also available at http://subversion.tigris.org/license-1.html.
-# If newer versions of this license are posted there, you may use a
-# newer version instead, at your option.
+#      http://www.apache.org/licenses/LICENSE-2.0
 #
+#    Unless required by applicable law or agreed to in writing,
+#    software distributed under the License is distributed on an
+#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+#    KIND, either express or implied.  See the License for the
+#    specific language governing permissions and limitations
+#    under the License.
 ######################################################################
 
 # General modules
-import sys, re, os, time
+import sys, re, os, time, subprocess
+import datetime
 
 # Our testing module
 import svntest
-from svntest import wc
+from svntest import wc, actions
+import logging
 
 # (abbreviation)
-Skip = svntest.testcase.Skip
-XFail = svntest.testcase.XFail
+Skip = svntest.testcase.Skip_deco
+SkipUnless = svntest.testcase.SkipUnless_deco
+XFail = svntest.testcase.XFail_deco
+Issues = svntest.testcase.Issues_deco
+Issue = svntest.testcase.Issue_deco
+Wimp = svntest.testcase.Wimp_deco
 Item = wc.StateItem
 
+logger = logging.getLogger()
+
 #----------------------------------------------------------------------
 # Helper function for testing stderr from co.
 # If none of the strings in STDERR list matches the regular expression
@@ -38,10 +54,9 @@ def test_stderr(re_string, stderr):
   for line in stderr:
     if exp_err_re.search(line):
       return
-  if svntest.main.verbose_mode:
-    for x in stderr:
-      sys.stdout.write(x)
-    print("Expected stderr reg-ex: '" + re_string + "'")
+  for x in stderr:
+    logger.debug(x[:-1])
+  logger.info("Expected stderr reg-ex: '" + re_string + "'")
   raise svntest.Failure("Checkout failed but not in the expected way")
 
 #----------------------------------------------------------------------
@@ -143,55 +158,224 @@ def make_local_tree(sbox, mod_files=False, add_unversioned=False):
 #----------------------------------------------------------------------
 
 def checkout_with_obstructions(sbox):
-  """co with obstructions should fail without --force"""
+  """co with obstructions conflicts without --force"""
 
   make_local_tree(sbox, False, False)
 
-  svntest.actions.run_and_verify_svn("No error where some expected",
-                                     None, svntest.verify.AnyOutput,
-                                     "co", sbox.repo_url, sbox.wc_dir)
+  #svntest.factory.make(sbox,
+  #       """# Checkout with unversioned obstructions lying around.
+  #          svn co url wc_dir
+  #          svn status""")
+  #svntest.factory.make(sbox,
+  #       """# Now see to it that we can recover from the obstructions.
+  #          rm -rf A iota
+  #          svn up""")
+  #exit(0)
+
+  wc_dir = sbox.wc_dir
+  url = sbox.repo_url
+
+  # Checkout with unversioned obstructions causes tree conflicts.
+  # svn co url wc_dir
+  expected_output = svntest.wc.State(wc_dir, {
+    'iota'              : Item(status='  ', treeconflict='C'),
+    'A'                 : Item(status='  ', treeconflict='C'),
+    # And the updates below the tree conflict
+    'A/D'               : Item(status='  ', treeconflict='A'),
+    'A/D/gamma'         : Item(status='  ', treeconflict='A'),
+    'A/D/G'             : Item(status='  ', treeconflict='A'),
+    'A/D/G/rho'         : Item(status='  ', treeconflict='A'),
+    'A/D/G/pi'          : Item(status='  ', treeconflict='A'),
+    'A/D/G/tau'         : Item(status='  ', treeconflict='A'),
+    'A/D/H'             : Item(status='  ', treeconflict='A'),
+    'A/D/H/chi'         : Item(status='  ', treeconflict='A'),
+    'A/D/H/omega'       : Item(status='  ', treeconflict='A'),
+    'A/D/H/psi'         : Item(status='  ', treeconflict='A'),
+    'A/B'               : Item(status='  ', treeconflict='A'),
+    'A/B/E'             : Item(status='  ', treeconflict='A'),
+    'A/B/E/beta'        : Item(status='  ', treeconflict='A'),
+    'A/B/E/alpha'       : Item(status='  ', treeconflict='A'),
+    'A/B/F'             : Item(status='  ', treeconflict='A'),
+    'A/B/lambda'        : Item(status='  ', treeconflict='A'),
+    'A/C'               : Item(status='  ', treeconflict='A'),
+    'A/mu'              : Item(status='  ', treeconflict='A'),
+  })
+
+  expected_disk = svntest.main.greek_state.copy()
+  expected_disk.remove('A/B', 'A/B/E', 'A/B/E/beta', 'A/B/E/alpha', 'A/B/F',
+    'A/B/lambda', 'A/D', 'A/D/G', 'A/D/G/rho', 'A/D/G/pi', 'A/D/G/tau',
+    'A/D/H', 'A/D/H/psi', 'A/D/H/omega', 'A/D/H/chi', 'A/D/gamma', 'A/C')
+
+  actions.run_and_verify_checkout2(False, url, wc_dir, expected_output,
+    expected_disk, None, None, None, None)
+
+  # svn status
+  expected_status = actions.get_virginal_state(wc_dir, 1)
+  # A and iota are tree conflicted and obstructed
+  expected_status.tweak('A', 'iota', status='D ', wc_rev=1,
+                        treeconflict='C')
+
+  expected_status.tweak('A/D', 'A/D/G', 'A/D/G/rho', 'A/D/G/pi', 'A/D/G/tau',
+    'A/D/H', 'A/D/H/chi', 'A/D/H/omega', 'A/D/H/psi', 'A/D/gamma', 'A/B',
+    'A/B/E', 'A/B/E/beta', 'A/B/E/alpha', 'A/B/F', 'A/B/lambda', 'A/C',
+    status='D ')
+  # A/mu exists on disk, but is deleted
+  expected_status.tweak('A/mu', status='D ')
+
+  actions.run_and_verify_unquiet_status(wc_dir, expected_status)
+
+
+  # Now see to it that we can recover from the obstructions.
+  # rm -rf A iota
+  svntest.main.safe_rmtree( os.path.join(wc_dir, 'A') )
+  os.remove( os.path.join(wc_dir, 'iota') )
+
+
+  svntest.main.run_svn(None, 'revert', '-R', os.path.join(wc_dir, 'A'),
+                       os.path.join(wc_dir, 'iota'))
+
+  # svn up
+  expected_output = svntest.wc.State(wc_dir, {
+  })
+
+  expected_disk = svntest.main.greek_state.copy()
+
+  expected_status = actions.get_virginal_state(wc_dir, 1)
+
+  actions.run_and_verify_update(wc_dir, expected_output, expected_disk,
+    expected_status, None, None, None, None, None, False, wc_dir)
+
+
 
 #----------------------------------------------------------------------
 
 def forced_checkout_of_file_with_dir_obstructions(sbox):
-  """forced co fails if a dir obstructs a file"""
+  """forced co flags conflict if a dir obstructs a file"""
+  # svntest.factory.make(sbox,
+  #                      """mkdir $WC_DIR.other/iota
+  #                         svn co --force url $WC_DIR.other """)
+  sbox.build()
+  url = sbox.repo_url
+  wc_dir_other = sbox.add_wc_path('other')
 
-  make_local_tree(sbox, False, False)
+  other_iota = os.path.join(wc_dir_other, 'iota')
 
-  # Make the "other" working copy
-  other_wc = sbox.add_wc_path('other')
-  os.mkdir(other_wc)
-  os.mkdir(os.path.join(other_wc, "iota"))
+  # mkdir $WC_DIR.other/iota
+  os.makedirs(other_iota)
 
-  # Checkout the standard greek repos into a directory that has a dir named
-  # "iota" obstructing the file "iota" in the repos.  This should fail.
-  exit_code, sout, serr = svntest.actions.run_and_verify_svn(
-    "Expected error during co", None, svntest.verify.AnyOutput,
-    "co", "--force", sbox.repo_url, other_wc)
+  # svn co --force url $WC_DIR.other
+  expected_output = svntest.wc.State(wc_dir_other, {
+    'A'                 : Item(status='A '),
+    'A/B'               : Item(status='A '),
+    'A/B/E'             : Item(status='A '),
+    'A/B/E/alpha'       : Item(status='A '),
+    'A/B/E/beta'        : Item(status='A '),
+    'A/B/F'             : Item(status='A '),
+    'A/B/lambda'        : Item(status='A '),
+    'A/D'               : Item(status='A '),
+    'A/D/H'             : Item(status='A '),
+    'A/D/H/chi'         : Item(status='A '),
+    'A/D/H/omega'       : Item(status='A '),
+    'A/D/H/psi'         : Item(status='A '),
+    'A/D/G'             : Item(status='A '),
+    'A/D/G/pi'          : Item(status='A '),
+    'A/D/G/rho'         : Item(status='A '),
+    'A/D/G/tau'         : Item(status='A '),
+    'A/D/gamma'         : Item(status='A '),
+    'A/C'               : Item(status='A '),
+    'A/mu'              : Item(status='A '),
+    'iota'              : Item(status='  ', treeconflict='C'),
+  })
+
+  expected_disk = svntest.main.greek_state.copy()
+  expected_disk.tweak('iota', contents=None)
+
+  actions.run_and_verify_checkout(url, wc_dir_other, expected_output,
+    expected_disk, None, None, None, None, '--force')
 
-  test_stderr(".*Failed to add file.*a non-file object of the same name " \
-              "already exists", serr)
 
 #----------------------------------------------------------------------
 
 def forced_checkout_of_dir_with_file_obstructions(sbox):
-  """forced co fails if a file obstructs a dir"""
+  """forced co flags conflict if a file obstructs a dir"""
 
   make_local_tree(sbox, False, False)
 
-  # Make the "other" working copy
-  other_wc = sbox.add_wc_path('other')
-  os.mkdir(other_wc)
-  svntest.main.file_append(os.path.join(other_wc, "A"), "The file A")
+  #svntest.factory.make(sbox,"""
+  #          mkdir wc_dir_other
+  #          echo "The file A" > wc_dir_other/A
+  #          svn co --force url wc_dir_other
+  #          """)
+  #svntest.factory.make(sbox,"""
+  #          # Now see to it that we can recover from the obstructions.
+  #          rm wc_dir_other/A
+  #          svn up wc_dir_other""")
+  #exit(0)
 
-  # Checkout the standard greek repos into a directory that has a file named
-  # "A" obstructing the dir "A" in the repos.  This should fail.
-  exit_code, sout, serr = svntest.actions.run_and_verify_svn(
-    "Expected error during co", None, svntest.verify.AnyOutput,
-    "co", "--force", sbox.repo_url, other_wc)
+  url = sbox.repo_url
+  wc_dir_other = sbox.add_wc_path('other')
+
+  other_A = os.path.join(wc_dir_other, 'A')
+
+  # mkdir wc_dir_other
+  os.makedirs(wc_dir_other)
+
+  # echo "The file A" > wc_dir_other/A
+  svntest.main.file_write(other_A, 'The file A\n')
+
+  # svn co --force url wc_dir_other
+  expected_output = svntest.wc.State(wc_dir_other, {
+    'iota'              : Item(status='A '),
+    'A'                 : Item(status='  ', treeconflict='C'),
+    # And what happens below A
+    'A/mu'              : Item(status='  ', treeconflict='A'),
+    'A/D'               : Item(status='  ', treeconflict='A'),
+    'A/D/G'             : Item(status='  ', treeconflict='A'),
+    'A/D/G/tau'         : Item(status='  ', treeconflict='A'),
+    'A/D/G/pi'          : Item(status='  ', treeconflict='A'),
+    'A/D/G/rho'         : Item(status='  ', treeconflict='A'),
+    'A/D/H'             : Item(status='  ', treeconflict='A'),
+    'A/D/H/psi'         : Item(status='  ', treeconflict='A'),
+    'A/D/H/omega'       : Item(status='  ', treeconflict='A'),
+    'A/D/H/chi'         : Item(status='  ', treeconflict='A'),
+    'A/D/gamma'         : Item(status='  ', treeconflict='A'),
+    'A/C'               : Item(status='  ', treeconflict='A'),
+    'A/B'               : Item(status='  ', treeconflict='A'),
+    'A/B/E'             : Item(status='  ', treeconflict='A'),
+    'A/B/E/beta'        : Item(status='  ', treeconflict='A'),
+    'A/B/E/alpha'       : Item(status='  ', treeconflict='A'),
+    'A/B/F'             : Item(status='  ', treeconflict='A'),
+    'A/B/lambda'        : Item(status='  ', treeconflict='A'),
+  })
+
+  expected_disk = svntest.main.greek_state.copy()
+  expected_disk.remove('A/B', 'A/B/E', 'A/B/E/beta', 'A/B/E/alpha', 'A/B/F',
+    'A/B/lambda', 'A/D', 'A/D/G', 'A/D/G/rho', 'A/D/G/pi', 'A/D/G/tau',
+    'A/D/H', 'A/D/H/psi', 'A/D/H/omega', 'A/D/H/chi', 'A/D/gamma', 'A/mu',
+    'A/C')
+  expected_disk.tweak('A', contents='The file A\n')
+
+  actions.run_and_verify_checkout(url, wc_dir_other, expected_output,
+    expected_disk, None, None, None, None, '--force')
+
+
+  # Now see to it that we can recover from the obstructions.
+  # rm wc_dir_other/A
+  os.remove(other_A)
+
+  # svn up wc_dir_other
+  expected_output = svntest.wc.State(wc_dir_other, {
+  })
+
+  expected_disk = svntest.main.greek_state.copy()
+
+  expected_status = actions.get_virginal_state(wc_dir_other, 1)
+
+  svntest.main.run_svn(None, 'revert', '-R', os.path.join(wc_dir_other, 'A'))
+
+  actions.run_and_verify_update(wc_dir_other, expected_output, expected_disk,
+    expected_status, None, None, None, None, None, False, wc_dir_other)
 
-  test_stderr(".*Failed to add directory.*a non-directory object of the " \
-              "same name already exists", serr)
 
 #----------------------------------------------------------------------
 
@@ -292,19 +476,27 @@ def forced_checkout_with_versioned_obstruction(sbox):
   # Checkout the entire first repos into the fresh dir.  This should
   # fail because A is already checked out.  (Ideally, we'd silently
   # incorporate A's working copy into its parent working copy.)
-  exit_code, sout, serr = svntest.actions.run_and_verify_svn(
-    "Expected error during co", None, svntest.verify.AnyOutput,
-    "co", "--force", repo_url, fresh_wc_dir)
-
-  test_stderr("Failed to add directory '.*A'.*already exists", serr)
+  expected_output = svntest.wc.State(fresh_wc_dir, {
+    'iota'              : Item(status='A '),
+    'A'                 : Item(verb='Skipped'),
+  })
+  expected_wc = svntest.main.greek_state.copy()
+  svntest.actions.run_and_verify_checkout(repo_url, fresh_wc_dir,
+                                          expected_output, expected_wc,
+                                          None, None, None, None,
+                                          '--force')
 
   # Checkout the entire first repos into the other dir.  This should
   # fail because it's a different repository.
-  exit_code, sout, serr = svntest.actions.run_and_verify_svn(
-    "Expected error during co", None, svntest.verify.AnyOutput,
-    "co", "--force", repo_url, other_wc_dir)
-
-  test_stderr("UUID mismatch: existing directory '.*A'", serr)
+  expected_output = svntest.wc.State(other_wc_dir, {
+    'iota'              : Item(status='A '),
+    'A'                 : Item(verb='Skipped'),
+  })
+  expected_wc = svntest.main.greek_state.copy()
+  svntest.actions.run_and_verify_checkout(repo_url, other_wc_dir,
+                                          expected_output, expected_wc,
+                                          None, None, None, None,
+                                          '--force')
 
   #ensure that other_wc_dir_A is not affected by this forced checkout.
   svntest.actions.run_and_verify_svn("empty status output", None,
@@ -388,6 +580,7 @@ def import_and_checkout(sbox):
 
 #----------------------------------------------------------------------
 # Issue #2529.
+@Issue(2529)
 def checkout_broken_eol(sbox):
   "checkout file with broken eol style"
 
@@ -461,17 +654,35 @@ def checkout_peg_rev(sbox):
 
 #----------------------------------------------------------------------
 # Issue 2602: Test that peg revision dates are correctly supported.
+@Issue(2602)
 def checkout_peg_rev_date(sbox):
   "checkout with peg revision date"
 
   sbox.build()
   wc_dir = sbox.wc_dir
 
-  # note the current time to use it as peg revision date.
-  current_time = time.strftime("%Y-%m-%dT%H:%M:%S")
+  ## Get svn:date.
+  exit_code, output, errput = svntest.main.run_svn(None, 'propget', 'svn:date',
+                                                   '--revprop', '-r1',
+                                                   '--strict',
+                                                   sbox.repo_url)
+  if exit_code or errput != [] or len(output) != 1:
+    raise svntest.Failure("svn:date propget failed")
+  r1_string = output[0]
 
-  # sleep till the next second.
-  time.sleep(1.1)
+  ## Increment the svn:date date by one microsecond.
+  # TODO: pass tzinfo=UTC to datetime.datetime()
+  date_pattern = re.compile(r'(\d+)-(\d+)-(\d+)T(\d\d):(\d\d):(\d\d)\.(\d+)Z$')
+  r1_time = datetime.datetime(*map(int, date_pattern.match(r1_string).groups()))
+  peg_time = r1_time + datetime.timedelta(microseconds=1)
+  assert r1_time != peg_time
+  # peg_string is, by all likelihood, younger than r1's svn:date and older than
+  # r2's svn:date.  It is also not equal to either of them, so we test the
+  # binary search of svn:date values.
+  peg_string = '%04d-%02d-%02dT%02d:%02d:%02d.%06dZ' % \
+               tuple(getattr(peg_time, x)
+                     for x in ["year", "month", "day", "hour", "minute",
+                               "second", "microsecond"])
 
   # create a new revision
   mu_path = os.path.join(wc_dir, 'A', 'mu')
@@ -480,7 +691,7 @@ def checkout_peg_rev_date(sbox):
   svntest.actions.run_and_verify_svn(None, None, [],
                                      'ci', '-m', 'changed file mu', wc_dir)
 
-  # now checkout the repo@current_time in another folder, this should create our
+  # now checkout the repo@peg_string in another folder, this should create our
   # initial wc without the change in mu.
   checkout_target = sbox.add_wc_path('checkout')
   os.mkdir(checkout_target)
@@ -493,7 +704,24 @@ def checkout_peg_rev_date(sbox):
 
   # use an old date to checkout, that way we're sure we get the first revision
   svntest.actions.run_and_verify_checkout(sbox.repo_url +
-                                          '@{' + current_time + '}',
+                                          '@{' + peg_string + '}',
+                                          checkout_target,
+                                          expected_output,
+                                          expected_wc)
+
+  # now try another checkout with repo@r1_string 
+  checkout_target = sbox.add_wc_path('checkout2')
+  os.mkdir(checkout_target)
+
+  expected_output = svntest.main.greek_state.copy()
+  expected_output.wc_dir = checkout_target
+  expected_output.tweak(status='A ', contents=None)
+
+  expected_wc = svntest.main.greek_state.copy()
+
+  # use an old date to checkout, that way we're sure we get the first revision
+  svntest.actions.run_and_verify_checkout(sbox.repo_url +
+                                          '@{' + r1_string + '}',
                                           checkout_target,
                                           expected_output,
                                           expected_wc)
@@ -728,7 +956,8 @@ def co_with_obstructing_local_adds(sbox):
     'A/D/M/psi'     : Item(status='  ', copied='+', wc_rev='-'),
     'A/D/M/chi'     : Item(status='  ', copied='+', wc_rev='-'),
     'A/D/M/omega'   : Item(status='  ', copied='+', wc_rev='-'),
-    'A/D/M/I'       : Item(status='  ', copied='+', wc_rev='-'),
+    'A/D/M/I'       : Item(status='A ', copied='+', wc_rev='-',
+                           entry_status='  '), # A/D/MI is a new op_root
     'A/D/M/I/J'     : Item(status='  ', copied='+', wc_rev='-'),
     'A/D/M/I/K'     : Item(status='  ', copied='+', wc_rev='-'),
     'A/D/M/I/K/xi'  : Item(status='  ', copied='+', wc_rev='-'),
@@ -739,6 +968,9 @@ def co_with_obstructing_local_adds(sbox):
 
   expected_output = wc.State(wc_dir, {
     'A/D/M'         : Item(status='  ', treeconflict='C'),
+    'A/D/M/rho'     : Item(status='  ', treeconflict='A'),
+    'A/D/M/pi'      : Item(status='  ', treeconflict='A'),
+    'A/D/M/tau'     : Item(status='  ', treeconflict='A'),
     })
   expected_disk = wc.State('', {
     'gamma'     : Item("This is the file 'gamma'.\n"),
@@ -772,7 +1004,7 @@ def co_with_obstructing_local_adds(sbox):
                                           None, None, None, None,
                                           '--force')
 
-  expected_status.tweak('A/D/M', treeconflict='C')
+  expected_status.tweak('A/D/M', treeconflict='C', status='R ')
   expected_status.tweak(
     'A/D',
     'A/D/G',
@@ -792,6 +1024,9 @@ def co_with_obstructing_local_adds(sbox):
     'A/D/H/chi'      : Item(status='  ', wc_rev=4),
     'A/D/H/psi'      : Item(status='  ', wc_rev=4),
     'A/D/H/omega'    : Item(status='  ', wc_rev=4),
+    'A/D/M/pi'       : Item(status='D ', wc_rev=4),
+    'A/D/M/rho'      : Item(status='D ', wc_rev=4),
+    'A/D/M/tau'      : Item(status='D ', wc_rev=4),
     })
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
@@ -809,12 +1044,134 @@ def co_with_obstructing_local_adds(sbox):
                                           None, None, None, None,
                                           '--force')
 
-  expected_status.tweak('A/B/F/omicron', treeconflict='C')
+  expected_status.tweak('A/B/F/omicron', treeconflict='C', status='R ')
   expected_status.add({
     'A/B/F'         : Item(status='  ', wc_rev=4),
     })
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
+#----------------------------------------------------------------------
+# Test if checking out from a Windows driveroot is supported.
+def checkout_wc_from_drive(sbox):
+  "checkout from the root of a Windows drive"
+
+  def find_the_next_available_drive_letter():
+    "find the first available drive"
+
+    # get the list of used drive letters, use some Windows specific function.
+    try:
+      import win32api
+
+      drives=win32api.GetLogicalDriveStrings()
+      drives=drives.split('\000')
+
+      for d in range(ord('G'), ord('Z')+1):
+        drive = chr(d)
+        if not drive + ':\\' in drives:
+          return drive
+    except ImportError:
+      # In ActiveState python x64 win32api is not available
+      for d in range(ord('G'), ord('Z')+1):
+        drive = chr(d)
+        if not os.path.isdir(drive + ':\\'):
+          return drive
+
+    return None
+
+  # Skip the test if not on Windows
+  if not svntest.main.windows:
+    raise svntest.Skip
+
+  # just create an empty folder, we'll checkout later.
+  sbox.build(create_wc = False)
+  svntest.main.safe_rmtree(sbox.wc_dir)
+  os.mkdir(sbox.wc_dir)
+
+  # create a virtual drive to the repository folder
+  drive = find_the_next_available_drive_letter()
+  if drive is None:
+    raise svntest.Skip
+
+  subprocess.call(['subst', drive +':', sbox.repo_dir])
+  repo_url = 'file:///' + drive + ':/'
+  wc_dir = sbox.wc_dir
+  was_cwd = os.getcwd()
+
+  try:
+    expected_wc = svntest.main.greek_state.copy()
+    expected_output = wc.State(wc_dir, {
+      'A'                 : Item(status='A '),
+      'A/D'               : Item(status='A '),
+      'A/D/H'             : Item(status='A '),
+      'A/D/H/psi'         : Item(status='A '),
+      'A/D/H/chi'         : Item(status='A '),
+      'A/D/H/omega'       : Item(status='A '),
+      'A/D/G'             : Item(status='A '),
+      'A/D/G/tau'         : Item(status='A '),
+      'A/D/G/pi'          : Item(status='A '),
+      'A/D/G/rho'         : Item(status='A '),
+      'A/D/gamma'         : Item(status='A '),
+      'A/C'               : Item(status='A '),
+      'A/mu'              : Item(status='A '),
+      'A/B'               : Item(status='A '),
+      'A/B/E'             : Item(status='A '),
+      'A/B/E/alpha'       : Item(status='A '),
+      'A/B/E/beta'        : Item(status='A '),
+      'A/B/F'             : Item(status='A '),
+      'A/B/lambda'        : Item(status='A '),
+      'iota'              : Item(status='A '),
+    })
+    svntest.actions.run_and_verify_checkout(repo_url, wc_dir,
+                                            expected_output, expected_wc,
+                                            None, None, None, None)
+
+    wc2_dir = sbox.add_wc_path('2')
+    expected_output = wc.State(wc2_dir, {
+      'D'                 : Item(status='A '),
+      'D/H'               : Item(status='A '),
+      'D/H/psi'           : Item(status='A '),
+      'D/H/chi'           : Item(status='A '),
+      'D/H/omega'         : Item(status='A '),
+      'D/G'               : Item(status='A '),
+      'D/G/tau'           : Item(status='A '),
+      'D/G/pi'            : Item(status='A '),
+      'D/G/rho'           : Item(status='A '),
+      'D/gamma'           : Item(status='A '),
+      'C'                 : Item(status='A '),
+      'mu'                : Item(status='A '),
+      'B'                 : Item(status='A '),
+      'B/E'               : Item(status='A '),
+      'B/E/alpha'         : Item(status='A '),
+      'B/E/beta'          : Item(status='A '),
+      'B/F'               : Item(status='A '),
+      'B/lambda'          : Item(status='A '),
+    })
+    svntest.actions.run_and_verify_checkout(repo_url + '/A', wc2_dir,
+                                            expected_output, None,
+                                            None, None, None, None)
+
+    wc3_dir = sbox.add_wc_path('3')
+    expected_output = wc.State(wc3_dir, {
+      'H'                 : Item(status='A '),
+      'H/psi'             : Item(status='A '),
+      'H/chi'             : Item(status='A '),
+      'H/omega'           : Item(status='A '),
+      'G'                 : Item(status='A '),
+      'G/tau'             : Item(status='A '),
+      'G/pi'              : Item(status='A '),
+      'G/rho'             : Item(status='A '),
+      'gamma'             : Item(status='A '),
+    })
+
+    svntest.actions.run_and_verify_checkout(repo_url + '/A/D', wc3_dir,
+                                            expected_output, None,
+                                            None, None, None, None)
+
+  finally:
+    os.chdir(was_cwd)
+    # cleanup the virtual drive
+    subprocess.call(['subst', '/D', drive +':'])
+
 #----------------------------------------------------------------------
 
 # list all tests here, starting with None:
@@ -832,6 +1189,7 @@ test_list = [ None,
               checkout_peg_rev,
               checkout_peg_rev_date,
               co_with_obstructing_local_adds,
+              checkout_wc_from_drive
             ]
 
 if __name__ == "__main__":
