@@ -3,29 +3,43 @@
 #  trans_tests.py:  testing eol conversion and keyword substitution
 #
 #  Subversion is a tool for revision control.
-#  See http://subversion.tigris.org for more information.
+#  See http://subversion.apache.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2000-2006 CollabNet.  All rights reserved.
+#    Licensed to the Apache Software Foundation (ASF) under one
+#    or more contributor license agreements.  See the NOTICE file
+#    distributed with this work for additional information
+#    regarding copyright ownership.  The ASF licenses this file
+#    to you under the Apache License, Version 2.0 (the
+#    "License"); you may not use this file except in compliance
+#    with the License.  You may obtain a copy of the License at
 #
-# This software is licensed as described in the file COPYING, which
-# you should have received as part of this distribution.  The terms
-# are also available at http://subversion.tigris.org/license-1.html.
-# If newer versions of this license are posted there, you may use a
-# newer version instead, at your option.
+#      http://www.apache.org/licenses/LICENSE-2.0
 #
+#    Unless required by applicable law or agreed to in writing,
+#    software distributed under the License is distributed on an
+#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+#    KIND, either express or implied.  See the License for the
+#    specific language governing permissions and limitations
+#    under the License.
 ######################################################################
 
 # General modules
-import os, re
+import os, re, logging, sys
+
+logger = logging.getLogger()
 
 # Our testing module
 import svntest
 from svntest import wc
 
 # (abbreviation)
-Skip = svntest.testcase.Skip
-XFail = svntest.testcase.XFail
+Skip = svntest.testcase.Skip_deco
+SkipUnless = svntest.testcase.SkipUnless_deco
+XFail = svntest.testcase.XFail_deco
+Issues = svntest.testcase.Issues_deco
+Issue = svntest.testcase.Issue_deco
+Wimp = svntest.testcase.Wimp_deco
 Item = svntest.wc.StateItem
 
 
@@ -80,13 +94,13 @@ def check_keywords(actual_kw, expected_kw, name):
   """A Helper function to compare two keyword lists"""
 
   if len(actual_kw) != len(expected_kw):
-    print "Keyword lists are different by size"
+    logger.warn("Keyword lists are different by size")
     raise svntest.Failure
 
   for i in range(0,len(actual_kw)):
     if actual_kw[i] != expected_kw[i]:
-      print '%s item %s, Expected: %s' % (name, i, expected_kw[i][:-1])
-      print '%s item %s, Got:      %s' % (name, i, actual_kw[i][:-1])
+      logger.warn('%s item %s, Expected: %s', name, i, expected_kw[i][:-1])
+      logger.warn('%s item %s, Got:      %s', name, i, actual_kw[i][:-1])
       raise svntest.Failure
 
 def setup_working_copy(wc_dir, value_len):
@@ -99,6 +113,8 @@ def setup_working_copy(wc_dir, value_len):
   global url_exp_path
   global id_unexp_path
   global id_exp_path
+  global header_unexp_path
+  global header_exp_path
   global bogus_keywords_path
   global embd_author_rev_unexp_path
   global embd_author_rev_exp_path
@@ -118,6 +134,8 @@ def setup_working_copy(wc_dir, value_len):
   url_exp_path = os.path.join(wc_dir, 'url_exp')
   id_unexp_path = os.path.join(wc_dir, 'id_unexp')
   id_exp_path = os.path.join(wc_dir, 'id_exp')
+  header_unexp_path = os.path.join(wc_dir, 'header_unexp')
+  header_exp_path = os.path.join(wc_dir, 'header_exp')
   bogus_keywords_path = os.path.join(wc_dir, 'bogus_keywords')
   embd_author_rev_unexp_path = os.path.join(wc_dir, 'embd_author_rev_unexp')
   embd_author_rev_exp_path = os.path.join(wc_dir, 'embd_author_rev_exp')
@@ -132,6 +150,8 @@ def setup_working_copy(wc_dir, value_len):
   svntest.main.file_append(url_exp_path, "$URL: blah $")
   svntest.main.file_append(id_unexp_path, "$Id$")
   svntest.main.file_append(id_exp_path, "$Id: blah $")
+  svntest.main.file_append(header_unexp_path, "$Header$")
+  svntest.main.file_append(header_exp_path, "$Header: blah $")
   svntest.main.file_append(bogus_keywords_path, "$Arthur$\n$Rev0$")
   svntest.main.file_append(embd_author_rev_unexp_path,
                            "one\nfish\n$Author$ two fish\n red $Rev$\n fish")
@@ -169,14 +189,15 @@ def setup_working_copy(wc_dir, value_len):
 ### Helper functions for setting/removing properties
 
 # Set the property keyword for PATH.  Turn on all possible keywords.
-# ### todo: Later, take list of keywords to set.
+### todo: Later, take list of keywords to set.
 def keywords_on(path):
   svntest.actions.run_and_verify_svn(None, None, [], 'propset',
-                                     "svn:keywords", "Author Rev Date URL Id",
+                                     "svn:keywords",
+                                     "Author Rev Date URL Id Header",
                                      path)
 
 # Delete property NAME from versioned PATH in the working copy.
-# ### todo: Later, take list of keywords to remove from the propval?
+### todo: Later, take list of keywords to remove from the propval?
 def keywords_off(path):
   svntest.actions.run_and_verify_svn(None, None, [], 'propdel',
                                      "svn:keywords", path)
@@ -215,6 +236,8 @@ def keywords_from_birth(sbox):
     'url_exp' : Item(status='A ', wc_rev=0),
     'id_unexp' : Item(status='A ', wc_rev=0),
     'id_exp' : Item(status='A ', wc_rev=0),
+    'header_unexp' : Item(status='A ', wc_rev=0),
+    'header_exp' : Item(status='A ', wc_rev=0),
     'bogus_keywords' : Item(status='A ', wc_rev=0),
     'embd_author_rev_unexp' : Item(status='A ', wc_rev=0),
     'embd_author_rev_exp' : Item(status='A ', wc_rev=0),
@@ -230,6 +253,8 @@ def keywords_from_birth(sbox):
   svntest.main.run_svn(None, 'add', url_exp_path)
   svntest.main.run_svn(None, 'add', id_unexp_path)
   svntest.main.run_svn(None, 'add', id_exp_path)
+  svntest.main.run_svn(None, 'add', header_unexp_path)
+  svntest.main.run_svn(None, 'add', header_exp_path)
   svntest.main.run_svn(None, 'add', bogus_keywords_path)
   svntest.main.run_svn(None, 'add', embd_author_rev_unexp_path)
   svntest.main.run_svn(None, 'add', embd_author_rev_exp_path)
@@ -246,6 +271,8 @@ def keywords_from_birth(sbox):
   keywords_on(url_exp_path)
   keywords_on(id_unexp_path)
   keywords_on(id_exp_path)
+  keywords_on(header_unexp_path)
+  keywords_on(header_exp_path)
   keywords_on(embd_author_rev_exp_path)
   keywords_on(fixed_length_keywords_path)
   keywords_on(id_with_space_path)
@@ -259,6 +286,8 @@ def keywords_from_birth(sbox):
     'url_exp' : Item(verb='Adding'),
     'id_unexp' : Item(verb='Adding'),
     'id_exp' : Item(verb='Adding'),
+    'header_unexp' : Item(verb='Adding'),
+    'header_exp' : Item(verb='Adding'),
     'bogus_keywords' : Item(verb='Adding'),
     'embd_author_rev_unexp' : Item(verb='Adding'),
     'embd_author_rev_exp' : Item(verb='Adding'),
@@ -275,8 +304,9 @@ def keywords_from_birth(sbox):
   fp = open(url_unexp_path, 'r')
   lines = fp.readlines()
   if not ((len(lines) == 1)
-          and (re.match("\$URL: (http|file|svn|svn\\+ssh)://", lines[0]))):
-    print "URL expansion failed for", url_unexp_path
+          and (re.match("\$URL: (http|https|file|svn|svn\\+ssh)://",
+                        lines[0]))):
+    logger.warn("URL expansion failed for %s", url_unexp_path)
     raise svntest.Failure
   fp.close()
 
@@ -284,8 +314,9 @@ def keywords_from_birth(sbox):
   fp = open(url_exp_path, 'r')
   lines = fp.readlines()
   if not ((len(lines) == 1)
-          and (re.match("\$URL: (http|file|svn|svn\\+ssh)://", lines[0]))):
-    print "URL expansion failed for", url_exp_path
+          and (re.match("\$URL: (http|https|file|svn|svn\\+ssh)://",
+                        lines[0]))):
+    logger.warn("URL expansion failed for %s", url_exp_path)
     raise svntest.Failure
   fp.close()
 
@@ -294,7 +325,7 @@ def keywords_from_birth(sbox):
   lines = fp.readlines()
   if not ((len(lines) == 1)
           and (re.match("\$Id: id_unexp", lines[0]))):
-    print "Id expansion failed for", id_exp_path
+    logger.warn("Id expansion failed for %s", id_exp_path)
     raise svntest.Failure
   fp.close()
 
@@ -303,7 +334,27 @@ def keywords_from_birth(sbox):
   lines = fp.readlines()
   if not ((len(lines) == 1)
           and (re.match("\$Id: id_exp", lines[0]))):
-    print "Id expansion failed for", id_exp_path
+    logger.warn("Id expansion failed for %s", id_exp_path)
+    raise svntest.Failure
+  fp.close()
+
+  # Make sure the unexpanded Header keyword got expanded correctly.
+  fp = open(header_unexp_path, 'r')
+  lines = fp.readlines()
+  if not ((len(lines) == 1)
+          and (re.match("\$Header: (https?|file|svn|svn\\+ssh)://.* jrandom",
+                        lines[0]))):
+    logger.warn("Header expansion failed for %s", header_unexp_path)
+    raise svntest.Failure
+  fp.close()
+
+  # Make sure the preexpanded Header keyword got reexpanded correctly.
+  fp = open(header_exp_path, 'r')
+  lines = fp.readlines()
+  if not ((len(lines) == 1)
+          and (re.match("\$Header: (https?|file|svn|svn\\+ssh)://.* jrandom",
+                        lines[0]))):
+    logger.warn("Header expansion failed for %s", header_exp_path)
     raise svntest.Failure
   fp.close()
 
@@ -342,8 +393,7 @@ def keywords_from_birth(sbox):
     '$URL::x%sx$\n' % (' ' * len(url_expand_test_data))
     ]
 
-  fp = open(os.path.join(wc_dir, svntest.main.get_admin_name(),
-                         'text-base', 'fixed_length_keywords.svn-base'), 'r')
+  fp = open(svntest.wc.text_base_path(fixed_length_keywords_path), 'r')
   actual_textbase_kw = fp.readlines()
   fp.close()
   check_keywords(actual_textbase_kw, kw_textbase, "text base")
@@ -353,7 +403,7 @@ def keywords_from_birth(sbox):
   lines = fp.readlines()
   if not ((len(lines) == 1)
           and (re.match("\$Id: .*id with space", lines[0]))):
-    print "Id expansion failed for", id_with_space_path
+    logger.warn("Id expansion failed for %s", id_with_space_path)
     raise svntest.Failure
   fp.close()
 
@@ -363,7 +413,7 @@ def keywords_from_birth(sbox):
   if not ((len(lines) == 1)
           and (re.match("\$Id: .*id_exp with_\$_sign [^$]* jrandom \$",
                         lines[0]))):
-    print "Id expansion failed for", id_exp_with_dollar_path
+    logger.warn("Id expansion failed for %s", id_exp_with_dollar_path)
 
     raise svntest.Failure
   fp.close()
@@ -402,6 +452,7 @@ def keywords_from_birth(sbox):
 def do_nothing(x, y):
   return 0
 
+@Issue(631)
 def update_modified_with_translation(sbox):
   "update modified file with eol-style 'native'"
 
@@ -490,7 +541,7 @@ def update_modified_with_translation(sbox):
 # contents are transmitted to the server during commit, and b) that
 # after the commit, the file and its text-base have been changed to
 # have the new line-ending style.
-
+@Issue(1085)
 def eol_change_is_text_mod(sbox):
   "committing eol-style change forces text send"
 
@@ -528,7 +579,7 @@ def eol_change_is_text_mod(sbox):
                                      'ci', '-m', 'log msg', foo_path)
 
   # check 2: do the files have the right contents now?
-  contents = svntest.main.file_read(foo_path, 'rb')
+  contents = open(foo_path, 'rb').read()
   if svntest.main.windows:
     if contents != "1\n2\n3\n4\n5\n6\n7\n8\n9\n":
       raise svntest.Failure
@@ -536,16 +587,15 @@ def eol_change_is_text_mod(sbox):
     if contents != "1\r\n2\r\n3\r\n4\r\n5\r\n6\r\n7\r\n8\r\n9\r\n":
       raise svntest.Failure
 
-  foo_base_path = os.path.join(wc_dir, svntest.main.get_admin_name(),
-                               'text-base', 'foo.svn-base')
-  base_contents = svntest.main.file_read(foo_base_path, 'rb')
+  foo_base_path = svntest.wc.text_base_path(foo_path)
+  base_contents = open(foo_base_path, 'rb').read()
   if contents != base_contents:
     raise svntest.Failure
 
 #----------------------------------------------------------------------
 # Regression test for issue #1151.  A single file in a directory
 # didn't get keywords expanded on checkout.
-
+@Issue(1151)
 def keyword_expanded_on_checkout(sbox):
   "keyword expansion for lone file in directory"
 
@@ -577,8 +627,9 @@ def keyword_expanded_on_checkout(sbox):
   fp = open(other_url_path, 'r')
   lines = fp.readlines()
   if not ((len(lines) == 1)
-          and (re.match("\$URL: (http|file|svn|svn\\+ssh)://", lines[0]))):
-    print "URL expansion failed for", other_url_path
+          and (re.match("\$URL: (http|https|file|svn|svn\\+ssh)://",
+                        lines[0]))):
+    logger.warn("URL expansion failed for %s", other_url_path)
     raise svntest.Failure
   fp.close()
 
@@ -713,9 +764,9 @@ def propset_commit_checkout_nocrash(sbox):
                                      sbox.repo_url,
                                      other_wc_dir)
 
-  mu_other_contents = svntest.main.file_read(mu_other_path)
+  mu_other_contents = open(mu_other_path).read()
   if mu_other_contents != "This is the file 'mu'.\n$Rev: 3 $":
-    print "'%s' does not have the expected contents" % mu_other_path
+    logger.warn("'%s' does not have the expected contents", mu_other_path)
     raise svntest.Failure
 
 
@@ -744,6 +795,165 @@ def propset_revert_noerror(sbox):
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
 
+def props_only_file_update(sbox):
+  "retranslation occurs on a props-only update"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  iota_path = os.path.join(wc_dir, 'iota')
+  content = ["This is the file 'iota'.\n",
+             "$Author$\n",
+             ]
+  content_expanded = ["This is the file 'iota'.\n",
+                      "$Author: jrandom $\n",
+                      ]
+
+  # Create r2 with iota's contents and svn:keywords modified
+  open(iota_path, 'w').writelines(content)
+  svntest.main.run_svn(None, 'propset', 'svn:keywords', 'Author', iota_path)
+
+  expected_output = wc.State(wc_dir, {
+    'iota' : Item(verb='Sending'),
+    })
+
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
+  expected_status.tweak('iota', wc_rev=2)
+
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        expected_output,
+                                        expected_status,
+                                        None,
+                                        wc_dir)
+
+  # Create r3 that drops svn:keywords
+
+  # put the content back to its untranslated form
+  open(iota_path, 'w').writelines(content)
+
+  svntest.main.run_svn(None, 'propdel', 'svn:keywords', iota_path)
+
+  expected_status.tweak('iota', wc_rev=3)
+
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        expected_output,
+                                        expected_status,
+                                        None,
+                                        wc_dir)
+
+  # Now, go back to r2. iota should have the Author keyword expanded.
+  expected_disk = svntest.main.greek_state.copy()
+  expected_disk.tweak('iota', contents=''.join(content_expanded))
+
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
+
+  svntest.actions.run_and_verify_update(wc_dir,
+                                        None, None, expected_status,
+                                        None,
+                                        None, None, None, None,
+                                        False,
+                                        wc_dir, '-r', '2')
+
+  if open(iota_path).read() != ''.join(content_expanded):
+    raise svntest.Failure("$Author$ is not expanded in 'iota'")
+
+  # Update to r3. this should retranslate iota, dropping the keyword expansion
+  expected_disk = svntest.main.greek_state.copy()
+  expected_disk.tweak('iota', contents=''.join(content))
+
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 3)
+
+  svntest.actions.run_and_verify_update(wc_dir,
+                                        None, expected_disk, expected_status,
+                                        None,
+                                        None, None, None, None,
+                                        False,
+                                        wc_dir)
+
+  if open(iota_path).read() != ''.join(content):
+    raise svntest.Failure("$Author$ is not contracted in 'iota'")
+
+  # We used to leave some temporary files around. Make sure that we don't.
+  temps = os.listdir(os.path.join(wc_dir, svntest.main.get_admin_name(), 'tmp'))
+  if os.path.exists(os.path.join(wc_dir, svntest.main.get_admin_name(),
+                                 'tmp', 'props')):
+    temps.remove('prop-base')
+    temps.remove('props')
+  if temps:
+    logger.warn('Temporary files leftover: %s', (', '.join(temps),))
+    raise svntest.Failure
+
+@XFail()
+@Issues(4327)
+def autoprops_inconsistent_eol(sbox):
+  "able to handle inconsistent eols on add"
+
+  sbox.build(read_only = True)
+  wc_dir = sbox.wc_dir
+
+  text = 'line with NL\n' + \
+         'line with CR\r' + \
+         'line with CRLF\r\n' + \
+         'line with LFCR (or is that not a line? ;-)\n\r'
+
+  # Compensate for python smartness
+  if sys.platform == 'win32':
+    expected_text = text.replace('\r\n', '\n')
+  else:
+    expected_text = text
+
+  sbox.simple_add_text(text, 'add.c')
+  sbox.simple_add_text(text, 'add-force.c')
+
+  svntest.actions.run_and_verify_svn(None, None, '.*inconsistent newlines.*',
+                                     'ps', 'svn:eol-style', 'native',
+                                     sbox.ospath('add.c'))
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ps', 'svn:eol-style', 'native', '--force',
+                                     sbox.ospath('add.c'))
+
+  expected_disk = svntest.main.greek_state.copy()
+
+  expected_disk.add({
+    'add-force.c'  : Item(contents=expected_text),
+    'add.c'        : Item(contents=expected_text),
+  })
+
+  # Verify that both add and add-force haven't been changed
+  svntest.actions.verify_disk(wc_dir, expected_disk)
+
+  sbox.simple_propset('svn:auto-props', '*.c = svn:eol-style=native', '')
+
+
+  svntest.main.file_write(sbox.ospath('auto.c'), text, mode='wb')
+
+  expected_output = ['A         %s\n' % sbox.ospath('auto.c')]
+
+  # Fails with svn: E200009: File '.*auto.c' has inconsistent newlines
+  svntest.actions.run_and_verify_svn(None, expected_output,
+                                     [], 'add', sbox.ospath('auto.c'))
+
+@XFail()
+@Issues(4327)
+def autoprops_inconsistent_mime(sbox):
+  "able to handle inconsistent mime on add"
+
+  sbox.build(read_only = True)
+
+  sbox.simple_propset('svn:auto-props',
+                      '*.c = svn:eol-style=native\n'
+                      'c.* = svn:mime-type=application/octet-stream', '')
+
+  sbox.simple_append('c.iota.c', '')
+
+  expected_output = ['A         %s\n' % sbox.ospath('c.iota.c')]
+
+  # Fails with svn: E200009: File '.*c.iota.c' has binary mime type property
+  svntest.actions.run_and_verify_svn(None, expected_output,
+                                     [], 'add', sbox.ospath('c.iota.c'))
+
+
 ########################################################################
 # Run the tests
 
@@ -761,6 +971,9 @@ test_list = [ None,
               copy_propset_commit,
               propset_commit_checkout_nocrash,
               propset_revert_noerror,
+              props_only_file_update,
+              autoprops_inconsistent_eol,
+              autoprops_inconsistent_mime,
              ]
 
 if __name__ == '__main__':
