@@ -1,13 +1,41 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
+#
+#
+# Licensed to the Apache Software Foundation (ASF) under one
+# or more contributor license agreements.  See the NOTICE file
+# distributed with this work for additional information
+# regarding copyright ownership.  The ASF licenses this file
+# to you under the Apache License, Version 2.0 (the
+# "License"); you may not use this file except in compliance
+# with the License.  You may obtain a copy of the License at
+#
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+#
+#
 
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.5.x/tools/hook-scripts/svnperms.py $
-# $LastChangedDate: 2007-08-26 01:04:48 +0000 (Sun, 26 Aug 2007) $
-# $LastChangedBy: hwright $
-# $LastChangedRevision: 26317 $
+# $HeadURL: http://svn.apache.org/repos/asf/subversion/branches/1.8.x/tools/hook-scripts/svnperms.py $
+# $LastChangedDate: 2011-07-12 18:37:44 +0000 (Tue, 12 Jul 2011) $
+# $LastChangedBy: blair $
+# $LastChangedRevision: 1145712 $
 
-import commands
 import sys, os
 import getopt
+import shlex
+
+try:
+  # Python >=3.0
+  from subprocess import getstatusoutput as subprocess_getstatusoutput
+except ImportError:
+  # Python <3.0
+  from commands import getstatusoutput as subprocess_getstatusoutput
 try:
     my_getopt = getopt.gnu_getopt
 except AttributeError:
@@ -35,7 +63,7 @@ class Config:
         cursectdict = None
         optname = None
         lineno = 0
-        for line in file.xreadlines():
+        for line in file:
             lineno = lineno + 1
             if line.isspace() or line[0] == '#':
                 continue
@@ -63,8 +91,8 @@ class Config:
                     self._sections_list.append((sectname, cursectlist))
                     optname = None
                 elif cursectdict is None:
-                    raise Error, "%s:%d: no section header" % \
-                                 (filename, lineno)
+                    raise Error("%s:%d: no section header" % \
+                                 (filename, lineno))
                 else:
                     m = OPTION.match(line)
                     if m:
@@ -73,14 +101,14 @@ class Config:
                         cursectdict[optname] = optval
                         cursectlist.append([optname, optval])
                     else:
-                        raise Error, "%s:%d: parsing error" % \
-                                     (filename, lineno)
+                        raise Error("%s:%d: parsing error" % \
+                                     (filename, lineno))
 
     def sections(self):
-        return self._sections_dict.keys()
+        return list(self._sections_dict.keys())
 
     def options(self, section):
-        return self._sections_dict.get(section, {}).keys()
+        return list(self._sections_dict.get(section, {}).keys())
 
     def get(self, section, option, default=None):
         return self._sections_dict.get(option, default)
@@ -102,7 +130,17 @@ class Permission:
 
     def parse_groups(self, groupsiter):
         for option, value in groupsiter:
-            self._group[option] = value.split()
+            groupusers = []
+            for token in shlex.split(value):
+                # expand nested groups in place; no forward decls
+                if token[0] == "@":
+                    try:
+                        groupusers.extend(self._group[token[1:]])
+                    except KeyError:
+                        raise Error, "group '%s' not found" % token[1:]
+                else:
+                    groupusers.append(token)
+            self._group[option] = groupusers
 
     def parse_perms(self, permsiter):
         for option, value in permsiter:
@@ -120,8 +158,8 @@ class Permission:
                         try:
                             users.extend(self._group[groupuser[1:]])
                         except KeyError:
-                            raise Error, "group '%s' not found" % \
-                                         groupuser[1:]
+                            raise Error("group '%s' not found" % \
+                                         groupuser[1:])
                     else:
                         users.append(groupuser)
                 self._permlist.append((pattern, users, perms))
@@ -141,12 +179,12 @@ class SVNLook:
 
     def _execcmd(self, *cmd, **kwargs):
         cmdstr = " ".join(cmd)
-        status, output = commands.getstatusoutput(cmdstr)
+        status, output = subprocess_getstatusoutput(cmdstr)
         if status != 0:
             sys.stderr.write(cmdstr)
             sys.stderr.write("\n")
             sys.stderr.write(output)
-            raise Error, "command failed: %s\n%s" % (cmdstr, output)
+            raise Error("command failed: %s\n%s" % (cmdstr, output))
         return status, output
 
     def _execsvnlook(self, cmd, *args, **kwargs):
@@ -156,18 +194,18 @@ class SVNLook:
         execcmd_kwargs = {}
         keywords = ["show", "noerror"]
         for key in keywords:
-            if kwargs.has_key(key):
+            if key in kwargs:
                 execcmd_kwargs[key] = kwargs[key]
         return self._execcmd(*execcmd_args, **execcmd_kwargs)
 
     def _add_txnrev(self, cmd_args, received_kwargs):
-        if received_kwargs.has_key("txn"):
+        if "txn" in received_kwargs:
             txn = received_kwargs.get("txn")
             if txn is not None:
                 cmd_args += ["-t", txn]
         elif self.txn is not None:
             cmd_args += ["-t", self.txn]
-        if received_kwargs.has_key("rev"):
+        if "rev" in received_kwargs:
             rev = received_kwargs.get("rev")
             if rev is not None:
                 cmd_args += ["-r", rev]
@@ -207,9 +245,9 @@ def check_perms(filename, section, repos, txn=None, rev=None, author=None):
     try:
         config = Config(filename)
     except IOError:
-        raise Error, "can't read config file "+filename
+        raise Error("can't read config file "+filename)
     if not section in config.sections():
-        raise Error, "section '%s' not found in config file" % section
+        raise Error("section '%s' not found in config file" % section)
     perm = Permission()
     perm.parse_groups(config.walk("groups"))
     perm.parse_groups(config.walk(section+" groups"))
@@ -231,7 +269,7 @@ def check_perms(filename, section, repos, txn=None, rev=None, author=None):
     if permerrors:
         permerrors.insert(0, "you don't have enough permissions for "
                              "this transaction:")
-        raise Error, "\n".join(permerrors)
+        raise Error("\n".join(permerrors))
 
 
 # Command:
@@ -247,7 +285,7 @@ Options:
     -s NAME    Use section NAME as permission section (default is
                repository name, extracted from repository path)
     -R REV     Query revision REV for commit information (for tests)
-    -A AUTHOR  Check commit as if AUTHOR had commited it (for tests)
+    -A AUTHOR  Check commit as if AUTHOR had committed it (for tests)
     -h         Show this message
 """
 
@@ -259,7 +297,7 @@ def parse_options():
     try:
         opts, args = my_getopt(sys.argv[1:], "f:s:r:t:R:A:h", ["help"])
     except getopt.GetoptError, e:
-        raise Error, e.msg
+        raise Error(e.msg)
     class Options: pass
     obj = Options()
     obj.filename = None
@@ -290,8 +328,7 @@ def parse_options():
     if not (obj.transaction or obj.revision):
         missingopts.append("either transaction or a revision")
     if missingopts:
-        raise MissingArgumentsException, \
-              "missing required option(s): " + ", ".join(missingopts)
+        raise MissingArgumentsException("missing required option(s): " + ", ".join(missingopts))
     obj.repository = os.path.abspath(obj.repository)
     if obj.filename is None:
         obj.filename = os.path.join(obj.repository, "conf", "svnperms.conf")
@@ -301,8 +338,8 @@ def parse_options():
             os.path.isdir(os.path.join(obj.repository, "db")) and
             os.path.isdir(os.path.join(obj.repository, "hooks")) and
             os.path.isfile(os.path.join(obj.repository, "format"))):
-        raise Error, "path '%s' doesn't look like a repository" % \
-                     obj.repository
+        raise Error("path '%s' doesn't look like a repository" % \
+                     obj.repository)
 
     return obj
 
